/****************************************************************************
**
** Copyright (C) 2015 Klaralvdalens Datakonsult AB (KDAB).
** Copyright (C) 2017 The Qt Company Ltd.
** Contact: http://www.qt.io/licensing/
**
** This file is part of Qt 3D Studio.
**
** $QT_BEGIN_LICENSE:GPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see https://www.qt.io/terms-conditions. For further
** information use the contact form at https://www.qt.io/contact-us.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3 or (at your option) any later version
** approved by the KDE Free Qt Foundation. The licenses are as published by
** the Free Software Foundation and appearing in the file LICENSE.GPL3
** included in the packaging of this file. Please review the following
** information to ensure the GNU General Public License requirements will
** be met: https://www.gnu.org/licenses/gpl-3.0.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QT3DRENDER_DRAGON_SHADERBUILDER_H
#define QT3DRENDER_DRAGON_SHADERBUILDER_H

//
//  W A R N I N G
//  -------------
//
// This file is not part of the Qt API.  It exists for the convenience
// of other Qt classes.  This header file may change from version to
// version without notice, or even be removed.
//
// We mean it.
//

#include <private/dragonbackendnode_p.h>
#include <Qt3DRender/private/qgraphicsapifilter_p.h>

#include <Qt3DCore/qpropertyupdatedchange.h>

QT_BEGIN_NAMESPACE

namespace Qt3DRender {

namespace Dragon {

class ShaderBuilder : public BackendNode
{
public:
    enum ShaderType {
        Vertex = 0,
        TessellationControl,
        TessellationEvaluation,
        Geometry,
        Fragment,
        Compute
    };

    static QString getPrototypesFile();
    static void setPrototypesFile(const QString &file);
    static QStringList getPrototypeNames();

    Qt3DCore::QNodeId shaderProgramId() const;
    QStringList enabledLayers() const;

    GraphicsApiFilterData graphicsApi() const;
    void setGraphicsApi(const GraphicsApiFilterData &graphicsApi);

    QUrl shaderGraph(ShaderType type) const;
    void setShaderGraph(ShaderType type, const QUrl &url);

    QByteArray shaderCode(ShaderType type) const;
    bool isShaderCodeDirty(ShaderType type) const;

    void generateCode(ShaderType type);

    void sceneChangeEvent(const Qt3DCore::QSceneChangePtr &e) override;
    void initializeFromPeer(const Qt3DCore::QNodeCreatedChangeBasePtr &change) final;

private:
    void setEnabledLayers(const QStringList &layers);

    GraphicsApiFilterData m_graphicsApi;
    Qt3DCore::QNodeId m_shaderProgramId;
    QStringList m_enabledLayers;
    QHash<ShaderType, QUrl> m_graphs;
    QHash<ShaderType, QByteArray> m_codes;
    QSet<ShaderType> m_dirtyTypes;
};

} // namespace Dragon
} // namespace Qt3DRender

QT_END_NAMESPACE

#endif // QT3DRENDER_DRAGON_SHADERBUILDER_H
